@@ -466,7 +466,7 @@ anx_read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   int cb_ret = ANX_CONTINUE;
 
   XTag * tag;
-  char * tag_name;
+  const char * tag_name;
   AnxClip * clip;
   AnxHead * head;
 
@@ -511,8 +511,10 @@ anx_read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
        */
       if (reader->read_head != anx_reader_stop_on_head) {
 	if (!reader->delivered_all_init_callbacks) {
-	  anx_reader_call_stream_callback (annodex);
-	  anx_reader_call_track_callbacks (annodex);
+	  cb_ret = anx_reader_call_stream_callback (annodex);
+	  if (cb_ret != ANX_CONTINUE) return cb_ret;
+	  cb_ret = anx_reader_call_track_callbacks (annodex);
+	  if (cb_ret != ANX_CONTINUE) return cb_ret;
 	  reader->delivered_all_init_callbacks = 1;
 	}
       }
@@ -1023,12 +1025,6 @@ anx_set_read_media_callback (ANNODEX * annodex, AnxReadMedia read_media)
   return ANX_ERR_DISABLED;
 }
 
-int
-anx_set_user_data (ANNODEX * annodex, void * user_data)
-{
-  return ANX_ERR_DISABLED;
-}
-
 long
 anx_read (ANNODEX * annodex, long n)
 {
