@@ -43,7 +43,10 @@
 
 #include "anx_private.h"
 
-/*#define DEBUG*/
+/* #define DEBUG */
+
+/* set to 0 or 1 */
+#define FLUSH_ALWAYS 0
 
 #define CMML_CONTENT_TYPE "text/x-cmml"
 #define EMPTY_CLIP "<clip/>"
@@ -342,6 +345,7 @@ anx_writer_update_eos (ANNODEX * annodex)
   AnxWriter * writer = &annodex->x.writer;
   AnxSource * media;
   AnxList * l;
+  int at_eos = 1;
 
   /* If we are at the end of the stream, mark media as done */
   for (l = writer->medias; l; l = l->next) {
@@ -350,10 +354,12 @@ anx_writer_update_eos (ANNODEX * annodex)
 #ifdef DEBUG
       printf ("anx_writer_update_eos: found one not at eos\n");
 #endif
+      at_eos = 0;
       break;
     }
   }
-  if (l == NULL) { /* if none found which are not at eos */
+
+  if (at_eos) {
 #ifdef DEBUG
     printf ("anx_writer_update_eos: all at eos\n");
 #endif
@@ -368,7 +374,7 @@ anx_packet_init (ANNODEX * annodex, unsigned char * buf, long n,
 		 long serialno, anx_int64_t granulepos, int bos, int eos,
 		 int flush)
 {
-  AnxWriter * writer = &annodex->x.writer;
+  /*AnxWriter * writer = &annodex->x.writer;*/
   ogg_packet op;
   int ret;
 
@@ -377,12 +383,8 @@ anx_packet_init (ANNODEX * annodex, unsigned char * buf, long n,
 	  n, serialno, bos);
 #endif
 
-#if 1
-  if (anx_media_eos (writer->current_media)) {
-    writer->done_media = 1;
-  }
-#else
-  anx_writer_update_eos (annodex);
+#if FLUSH_ALWAYS
+  flush |= OGGZ_FLUSH_AFTER;
 #endif
 
   annodex->current_granule = granulepos;
@@ -438,12 +440,6 @@ anx_init_mapping_packet (ANNODEX * annodex)
   *(ogg_int64_t *)b = _le_64 (td);
   b += 8;
 
-#ifdef DEBUG
-  printf ("anx_init_mapping_packet: timebase %lld/%lld\n[sizeof(AnxMapping) = %d\n",
-	  mapping->timebase_numerator, mapping->timebase_denominator,
-	  sizeof (*mapping));
-#endif
-
   return anx_packet_init (annodex, mapping, ANX_MAPPING_SIZE,
 			  annodex->mapping_serialno,
 			  0 /* granulepos */, 1 /* bos */, 0 /* eos */,
@@ -625,6 +621,10 @@ anx_writer_fill_media (ANNODEX * annodex)
   long read_len;
   long input_n;
 
+#ifdef DEBUG
+  int i = 0;
+#endif
+
   if (writer->media_buf_n != -1) {
 #ifdef DEBUG
     printf ("anx_writer_fill_media: media_buf_n != -1\n");
@@ -643,16 +643,14 @@ anx_writer_fill_media (ANNODEX * annodex)
 
   /* update the current_media */
   for (l = writer->medias; l; l = l->next) {
-#ifdef DEBUG
-    int i = 0;
-#endif
-
     media = (AnxSource *)l->data;
     media_time = anx_media_tell_time (media);
 #ifdef DEBUG
-    printf ("anx_write: got media [%d] %p time %f\n", i++, media, media_time);
+    printf ("anx_write: got media [%d] %p time %f %s\n", i++, media,
+	    media_time, anx_media_eos (media) ? "(eos)" : "");
 #endif
-    if (media_time != -1 && media_time < min_time) {
+    if (!anx_media_eos (media) &&
+	(media_time != -1 && media_time < min_time)) {
       soonest_media = media;
       min_time = media_time;
     }
@@ -702,16 +700,34 @@ static long
 anx_init_media_packet (ANNODEX * annodex, int flush)
 {
   AnxWriter * writer = &annodex->x.writer;
+  AnxWriterTrack * wsub;
   long input_n = writer->media_buf_n;
+  anx_int64_t granulepos;
+  int eos = 0;
 
   if (input_n == -1) return -1;
 
   writer->media_buf_n = -1;
+  granulepos = anx_media_tell (writer->current_media);
+  eos = anx_media_current_track_eos (writer->current_media);
+  wsub = writer->current_track;
+  if (wsub != NULL) {
+    if (wsub->written_eos) return -1;
+    if (eos) {
+      writer->current_track->written_eos = 1;
+    } else if (input_n == 0 && granulepos == wsub->last_granulepos) {
+      /* XXX: don't output duplicate gp zero-length packets */
+      return -1;
+    }
+    wsub->last_granulepos = granulepos;
+  }
+
+  if (granulepos >= 0) flush = OGGZ_FLUSH_AFTER;
 
   return anx_packet_init (annodex, writer->media_buf, input_n,
 			  writer->media_serialno,
-			  anx_media_tell (writer->current_media),
-			  0 /* bos */, 0 /* eos */, flush);
+			  granulepos,
+			  0 /* bos */, eos, flush);
 }
 
 static long
@@ -903,8 +919,13 @@ anx_writer_sync (ANNODEX * annodex)
 	  printf ("anx_writer_sync: writing eos for %s\n", 
 		  track->content_type);
 #endif
+#if 0
 	  granulepos = (anx_int64_t) anx_tell_time (annodex) * 
 	    track->granule_rate_n / track->granule_rate_d;
+#else
+	  /* Hack a fake ++granulepos for this zero-length packet */
+	  granulepos = wsub->last_granulepos+1;
+#endif
 	  anx_init_eos_packet (annodex, track->serialno, granulepos);
 	  wsub->written_eos = 1;
 	  made_packet = 1;
